@@ -17,7 +17,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -1479,7 +1479,7 @@ __again:
 	if (loop->play->access == loop->capt->access &&
 	    loop->play->format == loop->capt->format &&
 	    loop->play->rate == loop->capt->rate &&
-	    loop->play->channels == loop->play->channels &&
+	    loop->play->channels == loop->capt->channels &&
 	    loop->sync != SYNC_TYPE_SAMPLERATE) {
 		if (verbose > 1)
 			snd_output_printf(loop->output, "shared buffer!!!\n");
